#include <xc.h>
#include <stdio.h>
#include "eeprom_M24128.h"

#define _I2C_ON_OFF(x)		SSP1CON1bits.SSPEN = (x)
#define _I2C_START		SSP1CON2bits.SEN = 1
#define _I2C_START_BIT		SSP1CON2bits.SEN
#define _I2C_RESTART		SSP1CON2bits.RSEN = 1
#define _I2C_RESTART_BIT	SSP1CON2bits.RSEN
#define _I2C_STOP		SSP1CON2bits.PEN = 1
#define _I2C_STOP_BIT		SSP1CON2bits.PEN
#define _I2C_RECEIVE		SSP1CON2bits.RCEN = 1
#define _I2C_RECEIVE_BIT	SSP1CON2bits.RCEN
#define _I2C_ACK_NACK(x)	SSP1CON2bits.ACKDT = (x)
#define _I2C_ACK_NACK_SEND	SSP1CON2bits.ACKEN = 1
#define _SSP1IF_BIT		PIR1bits.SSP1IF
#define _SSP1IF_BIT_SET(x)	PIR1bits.SSP1IF = (x)
#define _SSP1BUF(x)		SSP1BUF = (x)
#define _SSP1BUF_RD		SSP1BUF
#define _SLAVE_ACK_BIT		SSP1CON2bits.ACKSTAT
#define _I2C_CKP(x)		SSP1CON1bits.CKP = (x)

#define IdleI2C()    while ((SSPCON2 & 0x1F) | (SSPSTATbits.R_nW))
#define NotAckI2C()  SSPCON2bits.ACKDT=1, SSPCON2bits.ACKEN=1;
#define AckI2C()     SSPCON2bits.ACKDT=0, SSPCON2bits.ACKEN=1;

#define I2C_CTRL_BYTE_WR 	(0xA0) // 0b10100000
#define I2C_CTRL_BYTE_RD 	(0xA1) // 0b10100001

#define MAX_ADDRESS		(0x3FFF)
#define MAX_PAGE_WRITE_BYTE	(64)

#define EEPROM_DELAY_WRITE_MS	(7)

#define RC_SUCCESS 		0
#define RC_FAIL 		1
#define RC_I2C_ACK_ERROR 	2

#define ON			1
#define OFF			0
#define HIGH			1
#define LOW			0

#define _XTAL_FREQ 8000000


#define MAX_BUF   5

void db_delay_ms(uint16_t ms)
{
	while (ms > 0) 
	{
		__delay_ms(1);
		ms--;
	}
}

int eeprom_M24128_write(uint16_t word_address, uint8_t *u8_data)
{  
   uint8_t lAddr = word_address & 0xff;
   uint8_t hAddr = word_address >> 8;

    // printf("EE-1\n\r");
	 // _SSP1IF_BIT_SET(LOW);		// clear SSP2 interrupt
	_I2C_ON_OFF(ON);		// enable the module
   
   // Write from hAddrlAddr
   _I2C_START;
   
   IdleI2C();   
   WriteI2C (0xA0);
   WriteI2C (hAddr);
   WriteI2C (lAddr);
   
   for (int i=0; i<MAX_BUF; i++)
      WriteI2C (u8_data[i]);
   
   _I2C_STOP;
   IdleI2C();   
   _I2C_ON_OFF(OFF);				// disable SSP2 module

	// mininum wait 5 ms
	db_delay_ms(EEPROM_DELAY_WRITE_MS);

   printf("EE-6\n\r");
	return RC_SUCCESS;
}

int eeprom_M24128_read(uint16_t word_address, uint8_t *u8_data)
{  uint8_t lAddr = word_address & 0xff;
   uint8_t hAddr = word_address >> 8;
    
   // Read from
   _I2C_ON_OFF(ON);		// enable the module
	_I2C_START;			// issue start condition
   IdleI2C();   
   WriteI2C (0xA0);
   WriteI2C (hAddr);
   WriteI2C (lAddr);
   _I2C_RESTART;
   IdleI2C();   
   WriteI2C (0xA1);
   for (int i=0; i<(MAX_BUF-1); i++)
    { u8_data[i] = ReadI2C();
      AckI2C();
      IdleI2C();           
    }
   // Last byte received
   u8_data[MAX_BUF-1] = ReadI2C();   
   NotAckI2C();   
   IdleI2C();           
    _I2C_STOP;			// issue STOP condition
   IdleI2C();   
   
   _I2C_ON_OFF(OFF);		// disable SSP2 module
	return RC_SUCCESS;
   
}

unsigned char WriteI2C(unsigned char data_out)
 { SSPBUF = data_out;            // write single byte to SSPBUF
   if (SSPCON1bits.WCOL)         // test if write collision occurred
      return (-1);               // if WCOL bit is set return negative #
   else
    { while(SSPSTATbits.BF);     // wait until write cycle is complete         
      IdleI2C();                 // ensure module is idle
      if (SSPCON2bits.ACKSTAT)   // test for ACK condition received
         return (-2);
      else 
         return (0);             // if WCOL bit is not set return non-negative #
    }
 }

unsigned char ReadI2C(void)
 { SSPCON2bits.RCEN = 1;         // enable master for 1 byte reception
   while (SSPSTATbits.BF==0);    // wait until byte received  
   return (SSPBUF);              // return with read byte 
 }
